#include "RealCal.hh" 
#include <iostream>	
#include <csignal>

using namespace std;

//Evil global variable
bool bRunning = true;

//Global terminate function, for use with catched signals	
void terminate (int iSig)
{
	cout << "Signal catched...terminating." << endl;
	bRunning = false;
}

//Here starts the implementation of the REALCAL-class
	
//Quit function, for use with the 'Quit'-button
void REALCAL::quit ()
{
	bRunning = false;
}

//Setup Resources
void REALCAL::setup_resources ()
{
	this->manager = CL_ComponentManager::create("RealCal.gui", false, this->style, this->gui);

     //Surfaces	

	//Boards

	this->boardsurface0 = CL_Surface::load("Surfaces/board0", this->resources);
	 
	//Borders
      this->bordersurface0 = CL_Surface::load("Surfaces/border0", this->resources);
	
     //GUI-stuff
	
	//Buttons
	this->button0 = (CL_Button *) this->manager->get_component("button0");
	this->button1 = (CL_Button *) this->manager->get_component("button1");

     //Connect buttons to slots
      	this->slot0 = this->button0->sig_clicked().connect(this, &REALCAL::open_window);
      	this->slot1 = this->button1->sig_clicked().connect(this, &REALCAL::quit);

     //Soundbuffers
	this->buffer0= CL_SoundBuffer::load("Sounds/sample", this->resources);	

	time_t currentTime;
 	tm localTime;
 	time(&currentTime); 
 	localTime=*(localtime(&currentTime));

	this->uiDay = localTime.tm_mday;
	this->uiMonth = localTime.tm_mon+1;

	cout << uiDay << ". " <<  uiMonth <<"." << endl;

	if(this->uiMonth==12)
	{
		this->bUnlocked = true;
	}
	else 
	{
		this->bUnlocked = false;
	}
}


char* REALCAL::get_title() { return "Real C  A  L"; }

	
int REALCAL::main (int argc, char **argv)
{
  
 //Exception handling with try and catch	
   try
  {	
	this->bupdate= true;

      //If called with argument w (like window), set bFullscreen to false
	bool bFullscreen=true;	
	if (argc > 1)
	{
  		if (*argv[1]=='w')
  		{
			bFullscreen=false;
  		}
  	}
	
   //Set startvalue for rand()-gen
	time_t sec;
	unsigned int k = time(&sec);
	srand (k);

   //Link signals to global terminate function
	signal(SIGTERM, terminate);
	signal(SIGINT, terminate);
	signal(SIGABRT, terminate);
	signal(SIGQUIT, terminate);
	signal(SIGHUP, terminate);
	signal(SIGCHLD, terminate);
   //Init
	CL_SetupCore::init();
	CL_SetupGUI::init();
	CL_SetupDisplay::init();
	CL_SetupSound::init();

   //Set Videomode to 800x600,16bpp
	CL_Display::set_videomode(800, 600, 16, bFullscreen, false, true);
 
   //Setup Managers
	CL_ResourceManager tmpresources("resources/RealCalResources.scr", false);
	CL_StyleManager_Default tmpstyle(&tmpresources);
	CL_GUIManager tmpgui(&tmpstyle);
	this->resources = &tmpresources;
	this->style = &tmpstyle; 
	this->gui = &tmpgui;

  //Setup resources
	this->setup_resources();   
		
   //Main loop..if an update is needed (bupdate=true), draw everything to screen and flip
   //Else just do the 'keep_alive'-stuff and sleep for 20ms

	while (bRunning && !CL_Keyboard::get_keycode(CL_KEY_ESCAPE))
	{	
		CL_System::keep_alive();

	     //If R is pressed, set 'this->bupdate' to true  
		if (CL_Keyboard::get_keycode(CL_KEY_R))
		{
			this->bupdate=true;
		}

		if (this->bupdate)
		{
			CL_Display::clear_display(1.00f, 0.10f, 0.10f);
      			this->boardsurface0->put_screen(1,1);
			this->bordersurface0->put_screen(600,1);
			this->gui->show();
			
			CL_Display::flip_display();
			this->bupdate=false;
		}

		CL_System::sleep(20);
	}

   //Deinit	
	CL_SetupSound::deinit();
	CL_SetupDisplay::deinit();
	CL_SetupGUI::deinit();
	CL_SetupCore::deinit();

	cout << "Quit!" << endl;
   }

 //Catch exceptions and send to console (stdout) 
   catch (CL_Error error) 
   {
	cout << "Exception caught: " << error.message.c_str() << endl;
	return -1;
   }

 return 0;
}

//Open todays window
void REALCAL::open_window ()
{	
	if(bUnlocked)
	{
	}
	else
	{
		this->buffer0->play();
	}
}
